// mxgpio.cpp : Defines the exported functions for the DLL application.
//
/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/
/* DA720-W7E 
   First release by Sean Hsieh 2016.05.30
*/

 
/*
*	 LED 1	:              IO Port 0x206 bit 0 	
*	 LED 2	:			   IO Port 0x206 bit 1 	
*	 LED 3	:              IO Port 0x206 bit 2 	
*	 LED 4	:			   IO Port 0x206 bit 3 	
*	 LED 5	:              IO Port 0x206 bit 4 	
*	 LED 6	:			   IO Port 0x206 bit 5 	
* 
*/

/*
*	 Relay control:        IO Port 0x204 bit 6 
* 
*/


#include "stdafx.h"
#include <windows.h>
#include <stdio.h>
#include "devices.h"
#include <winioctl.h>
#include "gpioctl_v1.0.h"
#include "WinIo.h"
#include "MxGeneralIo.h"
#include "mxgpio.h"


IOADDR_DEF  GPIODEF[] = {
						{LED_IO_ADDRESS , 0xFF, 0, 0, "LED1"},  
						{LED_IO_ADDRESS , 0xFF, 0, 1, "LED2"},  
						{LED_IO_ADDRESS , 0xFF, 0, 2, "LED3"},  
						{LED_IO_ADDRESS , 0xFF, 0, 3, "LED4"},  
						{LED_IO_ADDRESS , 0xFF, 0, 4, "LED5" },
						{LED_IO_ADDRESS , 0xFF, 0, 5, "LED5" },
						{RELAY_IO_ADDRESS , 0xFF, 0, 6, "RELAY"}, 
	   	   		  };

//IOADDR_DEF  GPIODEF[] = {
//						{IO_ADDRESS_GPIO , 0xFF, 0, 5, "LED2"}, //
//						{IO_ADDRESS_GPIO , 0xFF, 0, 0, "LED0(Reserve)"}, //
//						{IO_ADDRESS_GPIO , 0xFF, 0, 1, "LED1(Reserve)"},  //
//						{IO_ADDRESS_GPIO , 0xFF, 0, 2, "PWR1"},  //
//						{IO_ADDRESS_GPIO , 0xFF, 0, 4, "PWR2"},  //
//						{IO_ADDRESS_GPIO , 0xFF, 0, 3, "RST1"},  //
//						{IO_ADDRESS_GPIO , 0xFF, 0, 6, "RST2"},  //
//						{IO_ADDRESS_GPIO , 0xFF, 0, 7, "SIO_PWR_IGN"}  //
//	   	   		  };

//IOADDR_DEF  GPIODEF[] = {
//						{IO_ADDRESS_GPIO , 0xFF, 0, 4, "PWR1"},  //
//						{IO_ADDRESS_GPIO , 0xFF, 0, 6, "PWR2"},  //
//						{IO_ADDRESS_GPIO , 0xFF, 0, 5, "RST1"},  //
//						{IO_ADDRESS_GPIO , 0xFF, 0, 7, "RST2"},  //
//	   	   		  };

/*
typedef struct _IOADDR_DEF {
    unsigned long IOAddress;
	unsigned long DataFilter;
	unsigned long DataOffset;
	unsigned long DataBit;
  	char *Desc;
} IOADDR_DEF;
*/

#if ORIGIN_GPIO
#include "stdafx.h"
#include "MxGeneralIo.h"
#include <windows.h>
#include <stdio.h>
#include "mxgpio.h"

#define SIGNAL_LOW 0
#define SIGNAL_HIGH 1

#define GPIO_SIGNAL_ENABLE SIGNAL_LOW
#define GPIO_SIGNAL_DISABLE SIGNALL_HIGH

#define DEVICE_IO_TYPE		IO_TYPE_IOBASE
#define IO_ADDRESS_GPIO		0x301

typedef struct _GPIO {
	int	port;
	int	data;
} GPIO, *PGPIO;
#endif

#ifdef __cplusplus    // If used by C++ code, 
extern "C" {          // we need to export the C interface
#endif

/*	open a GPIO handle for later uses
	Inputs:
		none
	Returns:
		pointer to a GPIO handle. Return 0 on failure.
*/
HANDLE mxgpio_open(void)
{
	BOOL bResult;

	// Initialize WinIo
	bResult = MxGeneralIoOpen();
	if(bResult == FALSE)
	{
		printf("InitializeWinIo fail\n");
		return INVALID_HANDLE_VALUE;

	}

	return (HANDLE) 1;
}

/*	close a GPIO handle
	Inputs:
		<fd> the handle
	Returns:
		none
*/
void mxgpio_close(HANDLE fd)
{

	if(fd != INVALID_HANDLE_VALUE)
	{
		// Shut down WinIo
		MxGeneralIoClose();
	}
	
	return;
}

/*	set the mode of GPIO port
	Inputs:
		<fd>	the GPIO handle
		<port_no> the port number (started from 0)
		<mode>    1:Input, 0:Output
	Returns:
		0 on success, otherwise the function fails
*/
int mxgpio_set_mode(HANDLE fd, unsigned int port_no, int mode)
{
	return 0;
}

/*	get the mode of GPIO port
	Inputs:
		<fd>	the GPIO handle
		<port_no> the port number (started from 0)
	Returns:
		1 indicates Input, 0 indicates Output, otherwise the function fails
*/
int mxgpio_get_mode(HANDLE fd, unsigned int port_no)
{
	return 0;
}

/*	get the value (HIGH or LOW) of GPIO port
	Inputs:
		<fd>	the GPIO handle
		<port_no> the port number (started from 0)
		<data>    1:HIGH, 0:LOW
	Returns:
		0 on success, otherwise the function fails
*/
int mxgpio_set_data(HANDLE fd, unsigned int port, int data)
{
	GPIO gpio;
	DWORD dwBackSize=0;
	int ret=0;

	WORD	wPortAddr = (WORD)GPIODEF[port].IOAddress;
	DWORD	dwPortVal = 0;
	UCHAR	regData = 0x88;
	BOOL	bResult = false;
	
	/*
	 *	prepare the data to send
	 */
	gpio.port = port;
	gpio.data = data;

	/* Read current register value */
	bResult = MxGetGeneralIoVal(DEVICE_IO_TYPE,wPortAddr, &dwPortVal, 1);

	/* Shift the data to lower bits */
	regData=(UCHAR)dwPortVal;
	//printf("dwPortVal : 0x%x \n",dwPortVal);
	regData = regData & GPIODEF[port].DataFilter;
	
	/* Prepare the value */
	if ( gpio.data == SIGNAL_HIGH )
    {
		regData |= (1 << GPIODEF[port].DataBit);
    }
    else
    {
		regData &= ~(1 << GPIODEF[port].DataBit);
    }

	/* Set register */
	
	dwPortVal = regData;
	//printf("dwPortVal : 0x%x \n",dwPortVal);

	bResult = MxSetGeneralIoVal(DEVICE_IO_TYPE, wPortAddr, dwPortVal, 1);
	
	/*
	 * Wait for writing the data
	 */
	Sleep(50);
	
	if(bResult == FALSE)
	{
		ret=-1;
	}

	return ret;
}

/*	Get the value (HIGH or LOW) of GPIO port
	Inputs:
		<fd>	the GPIO handle
		<port_no> the port number (started from 0)
	Returns:
		1 indicates HIGH, 0 indicates LOW, otherwise the function fails
*/
int mxgpio_get_data(HANDLE fd, unsigned int port)
{
	GPIO	gpio;
	DWORD	dwBackSize = 0;
	DWORD	dwPortVal = 0x01;
	BOOL	bResult;
	WORD	wPortAddr = (WORD)GPIODEF[port].IOAddress;
	UCHAR	regData = 0x0;
	UCHAR	gpioData = 0x0;
	UCHAR	tempData = 0x0;
	UCHAR	checkData = 0x0;
	int		ret = 0;

	gpio.port = port;
	gpio.data = 0;

	/* Get register data */
	bResult = MxGetGeneralIoVal(DEVICE_IO_TYPE,wPortAddr, &dwPortVal, 1);

	/* extract the specified digit*/
	gpioData = (UCHAR)dwPortVal & (UCHAR)GPIODEF[port].DataFilter;
	//printf("mxdgio_get_input_signal : DataDIN1 = 0x%x \n",DataDIN);

	//printf("gpioData: 0x%x \n",gpioData);
	tempData =(1 << GPIODEF[port].DataBit);
	//printf("tempData: 0x%x \n",tempData);
	checkData = gpioData & tempData;
	//printf("checkData: 0x%x \n",checkData);


	if( (checkData-tempData)==0){
		//printf("true\n");
		gpio.data = SIGNAL_HIGH;
	} else {
		gpio.data = SIGNAL_LOW;
	}
	ret = gpio.data;
	
	/* 3.Check operation succeed */
	if(bResult == FALSE)
	{
		ret = -1;
	}

	return ret;
}


#if ORIGIN_GPIO
/*	open a GPIO handle for later uses
	Inputs:
		none
	Returns:
		pointer to a GPIO handle. Return 0 on failure.
*/
HANDLE mxgpio_open(void)
{
	BOOL bResult;

	// Initialize WinIo
	bResult = MxGeneralIoOpen();
	if(bResult == FALSE)
	{
		printf("InitializeWinIo fail\n");
		return INVALID_HANDLE_VALUE;

	}

	return (HANDLE) 1;
}

/*	close a GPIO handle
	Inputs:
		<fd> the handle
	Returns:
		none
*/
void mxgpio_close(HANDLE fd)
{

	if(fd != INVALID_HANDLE_VALUE)
	{
		// Shut down WinIo
		MxGeneralIoClose();
	}
	
	return;
}

/*	set the mode of GPIO port
	Inputs:
		<fd>	the GPIO handle
		<port_no> the port number (started from 0)
		<mode>    1:Input, 0:Output
	Returns:
		0 on success, otherwise the function fails
*/
int mxgpio_set_mode(HANDLE fd, unsigned int port_no, int mode)
{
	return 0;
}

/*	get the mode of GPIO port
	Inputs:
		<fd>	the GPIO handle
		<port_no> the port number (started from 0)
	Returns:
		1 indicates Input, 0 indicates Output, otherwise the function fails
*/
int mxgpio_get_mode(HANDLE fd, unsigned int port_no)
{
	return 0;
}

/*	get the value (HIGH or LOW) of GPIO port
	Inputs:
		<fd>	the GPIO handle
		<port_no> the port number (started from 0)
		<data>    1:HIGH, 0:LOW
	Returns:
		0 on success, otherwise the function fails
*/
int mxgpio_set_data(HANDLE fd, unsigned int port_no, int data)
{

	GPIO	gpio;
	DWORD	dwBackSize = 0;
	DWORD	dwPortVal = 0;
	WORD	wPortAddr = IO_ADDRESS_GPIO;
	BOOL	bResult;
	UCHAR	gpioData;
	int		ret = 0;

	gpio.port = port_no;
	gpio.data = data;

	/* 1.Get original value of LED register */
	bResult = MxGetGeneralIoVal(DEVICE_IO_TYPE,wPortAddr, &dwPortVal, 1);

	gpioData = (UCHAR)dwPortVal;

	/* 2.Calculate new value */
	if ( gpio.data == SIGNAL_HIGH )
	{
		gpioData |= (1 << gpio.port);
	}
	else
	{
		gpioData &= ~(1 << gpio.port);
	}

	/* 3.Set LED register */
	dwPortVal = gpioData;
	bResult = MxSetGeneralIoVal(DEVICE_IO_TYPE,wPortAddr,dwPortVal,1);

	/* 4.Wait for writing the data*/
	Sleep(50);
	
	if(bResult == FALSE)
	{
		ret=-1;
	}

	return ret;
}

/*	Get the value (HIGH or LOW) of GPIO port
	Inputs:
		<fd>	the GPIO handle
		<port_no> the port number (started from 0)
	Returns:
		1 indicates HIGH, 0 indicates LOW, otherwise the function fails
*/
int mxgpio_get_data(HANDLE fd, unsigned int port_no)
{
	GPIO	gpio;
	DWORD	dwBackSize = 0;
	DWORD	dwPortVal = 0;
	BOOL	bResult;
	WORD	wPortAddr = IO_ADDRESS_GPIO;
	UCHAR	gpioData;
	int		ret = 0;

	gpio.port = port_no;
	gpio.data = 0;

	/* 1.Get LED register */
	bResult = MxGetGeneralIoVal(DEVICE_IO_TYPE,wPortAddr, &dwPortVal, 1);

	/* 2.Calculate data of specified port */
	gpioData = (UCHAR)dwPortVal;
	if ( gpioData & (1 << gpio.port) )
	{
		gpio.data = SIGNAL_HIGH;
	}
	else
	{
		gpio.data = SIGNAL_LOW;
	}
	ret = gpio.data;
	
	/* 3.Check operation succeed */
	if(bResult == FALSE)
	{
		ret = -1;
	}

	return ret;
}
#endif

#ifdef __cplusplus    // If used by C++ code, 
}
#endif